@@ -1,9 +1,11 @@
 import logging
 import os
+import pathlib
+import re
 import shutil
 import sys
 from multiprocessing import Pool
-
+from sshkeyboard import listen_keyboard, stop_listening
 import keras
 import pandas as pd
 import lightkurve as lk
@@ -15,7 +17,9 @@ from astropy.coordinates import SkyCoord
 from astropy.wcs import WCS
 from astroquery.mast import Catalogs, Tesscut
 from keras import Sequential
-from keras.layers import Conv1D, Dropout, MaxPooling1D, Flatten, Dense, Embedding, LSTM
+from keras.layers import Conv1D, Dropout, MaxPooling1D, Flatten, Dense, Embedding, LSTM, ConvLSTM2D
+from lightkurve import TessTargetPixelFile
+from matplotlib.gridspec import GridSpec
 
 from sherlockpipe.ois.OisManager import OisManager
 
@@ -176,12 +180,59 @@ class MlTrainingSetPreparer:
         pix_coords = pix_coords
         return stars
 
+    def store_lc_data(self, lc_data, file):
+        centroids_x = lc_data["centroids_x"][~np.isnan(lc_data["centroids_x"])].to_numpy()
+        centroids_y = lc_data["centroids_y"][~np.isnan(lc_data["centroids_y"])].to_numpy()
+        motion_x = lc_data["motion_x"][~np.isnan(lc_data["motion_x"])].to_numpy()
+        motion_y = lc_data["motion_y"][~np.isnan(lc_data["motion_y"])].to_numpy()
+        dif = centroids_x[1:] - centroids_x[:-1]
+        jumps = np.where(np.abs(dif) > 1)[0]
+        jumps = np.append(jumps, len(lc_data))
+        previous_jump_index = 0
+        for jumpIndex in jumps:
+            centroids_x_token = lc_data["centroids_x"][previous_jump_index:jumpIndex]
+            lc_data["centroids_x"][previous_jump_index:jumpIndex] = centroids_x_token - np.nanmedian(centroids_x_token)
+            previous_jump_index = jumpIndex
+        previous_jump_index = 0
+        dif = centroids_y[1:] - centroids_y[:-1]
+        jumps = np.where(np.abs(dif) > 1)[0]
+        jumps = np.append(jumps, len(lc_data))
+        for jumpIndex in jumps:
+            centroids_y_token = lc_data["centroids_y"][previous_jump_index:jumpIndex]
+            lc_data["centroids_y"][previous_jump_index:jumpIndex] = centroids_y_token - np.nanmedian(centroids_y_token)
+            previous_jump_index = jumpIndex
+        previous_jump_index = 0
+        dif = motion_x[1:] - motion_x[:-1]
+        jumps = np.where(np.abs(dif) > 1)[0]
+        jumps = np.append(jumps, len(lc_data))
+        for jumpIndex in jumps:
+            motion_x_token = lc_data["motion_x"][previous_jump_index:jumpIndex]
+            lc_data["motion_x"][previous_jump_index:jumpIndex] = motion_x_token - np.nanmedian(motion_x_token)
+            previous_jump_index = jumpIndex
+        previous_jump_index = 0
+        dif = motion_y[1:] - motion_y[:-1]
+        jumps = np.where(np.abs(dif) > 1)[0]
+        jumps = np.append(jumps, len(lc_data))
+        for jumpIndex in jumps:
+            motion_y_token = lc_data["motion_y"][previous_jump_index:jumpIndex]
+            lc_data["motion_y"][previous_jump_index:jumpIndex] = motion_y_token - np.nanmedian(motion_y_token)
+            previous_jump_index = jumpIndex
+        lc_data.to_csv(file)
+        return lc_data
+
     def prepare_tic(self, prepare_tic_input):
         tic_id = str(prepare_tic_input.tic)
         target_dir = prepare_tic_input.dir + tic_id + "/"
         if not os.path.isdir(target_dir):
             os.mkdir(target_dir)
+        tpfs_short_dir = target_dir + "tpfs_short/"
+        tpfs_long_dir = target_dir + "tpfs_long/"
+        if not os.path.isdir(tpfs_short_dir):
+            os.mkdir(tpfs_short_dir)
+        if not os.path.isdir(tpfs_long_dir):
+            os.mkdir(tpfs_long_dir)
         lc_short = None
+        lc_data_short = None
         failed_target = None
         try:
             mission_lightcurve_builder = MissionLightcurveBuilder()
@@ -194,14 +245,11 @@ class MlTrainingSetPreparer:
             lcbuild_short = \
                 mission_lightcurve_builder.build(MissionObjectInfo(tic_id, 'all'), None, self.cache_dir)
             lc_short = lcbuild_short.lc
-            lc_data = lcbuild_short.lc_data
-            lc_data["centroids_x"] = lc_data["centroids_x"] - np.nanmedian(lc_data["centroids_x"])
-            lc_data["centroids_y"] = lc_data["centroids_y"] - np.nanmedian(lc_data["centroids_y"])
-            lc_data["motion_x"] = lc_data["motion_x"] - np.nanmedian(lc_data["motion_x"])
-            lc_data["motion_y"] = lc_data["motion_y"] - np.nanmedian(lc_data["motion_y"])
-            lc_data.to_csv(target_dir + "time_series_short.csv")
+            lc_data_short = self.store_lc_data(lcbuild_short.lc_data, target_dir + "time_series_short.csv")
             tpf_short = lk.search_targetpixelfile(tic_id, cadence="short", author="spoc").download_all(
                 download_dir=self.cache_dir + ".lightkurve-cache")
+            for tpf in tpf_short.data:
+                shutil.copy(tpf.path, tpfs_short_dir + os.path.basename(tpf.path))
             short_periodogram = lc_short.to_periodogram(oversample_factor=5)
             periodogram_df = pd.DataFrame(columns=['period', 'power'])
             periodogram_df["period"] = short_periodogram.period.value
@@ -240,20 +288,12 @@ class MlTrainingSetPreparer:
             star_df.to_csv(target_dir + "params_star.csv", index=False)
             sectors = lcbuild_long.sectors
             lc_long = lcbuild_long.lc
-            lc_data = lcbuild_long.lc_data
-            lc_data["centroids_x"] = lc_data["centroids_x"] - np.nanmedian(lc_data["centroids_x"])
-            lc_data["centroids_y"] = lc_data["centroids_y"] - np.nanmedian(lc_data["centroids_y"])
-            lc_data["motion_x"] = lc_data["motion_x"] - np.nanmedian(lc_data["motion_x"])
-            lc_data["motion_y"] = lc_data["motion_y"] - np.nanmedian(lc_data["motion_y"])
-            lc_data.to_csv(target_dir + "time_series_long.csv")
+            lc_data_long = self.store_lc_data(lcbuild_long.lc_data, target_dir + "time_series_long.csv")
             lcf_long = lc_long.remove_nans()
             tpf_long = lk.search_targetpixelfile(tic_id, cadence="long", author="tess-spoc")\
                 .download_all(download_dir=self.cache_dir + ".lightkurve-cache")
-            tpfs_dir = target_dir + "tpfs/"
-            if not os.path.exists(tpfs_dir):
-                os.mkdir(tpfs_dir)
             for tpf in tpf_long.data:
-                shutil.copy(tpf.path, tpfs_dir + os.path.basename(tpf.path))
+                shutil.copy(tpf.path, tpfs_long_dir + os.path.basename(tpf.path))
             long_periodogram = lc_long.to_periodogram(oversample_factor=5)
             periodogram_df = pd.DataFrame(columns=['period', 'power'])
             periodogram_df["period"] = long_periodogram.period.value
@@ -268,8 +308,9 @@ class MlTrainingSetPreparer:
             target_ois_df = pd.DataFrame(
                 columns=['id', 'name', 'period', 'period_err', 't0', 'to_err', 'depth', 'depth_err', 'duration',
                          'duration_err'])
-            tags_series_short = np.full(len(lc_short.time), "BL")
-            tags_series_long = np.full(len(lc_long.time), "BL")
+            if lc_data_short is not None:
+                tags_series_short = np.full(len(lc_data_short.time), "BL")
+            tags_series_long = np.full(len(lc_data_long.time), "BL")
             if prepare_tic_input.label is not None:
                 for index, row in target_ois.iterrows():
                     if row["OI"] not in prepare_tic_input.excluded_ois:
@@ -282,10 +323,10 @@ class MlTrainingSetPreparer:
                              "depth_err": row["Depth (ppm) err"], "duration": row["Duration (hours)"],
                              "duration_err": row["Duration (hours) err"]}, ignore_index=True)
                         if lc_short is not None:
-                            mask_short = tls.transit_mask(lc_short.time.value, row["Period (days)"],
+                            mask_short = tls.transit_mask(lc_data_short["time"].to_numpy(), row["Period (days)"],
                                                           row["Duration (hours)"] / 24, row["Epoch (BJD)"] - 2457000.0)
                             tags_series_short[mask_short] = prepare_tic_input.label
-                        mask_long = tls.transit_mask(lc_long.time.value, row["Period (days)"], row["Duration (hours)"] / 24,
+                        mask_long = tls.transit_mask(lc_data_long["time"].to_numpy(), row["Period (days)"], row["Duration (hours)"] / 24,
                                                      row["Epoch (BJD)"] - 2457000.0)
                         tags_series_long[mask_long] = prepare_tic_input.label
                 for index, row in prepare_tic_input.target_additional_ois_df.iterrows():
@@ -297,19 +338,18 @@ class MlTrainingSetPreparer:
                              "depth_err": row["Depth (ppm) err"], "duration": row["Duration (hours)"],
                              "duration_err": row["Duration (hours) err"]}, ignore_index=True)
                         if lc_short is not None:
-                            mask_short = tls.transit_mask(lc_short.time.value, row["Period (days)"],
+                            mask_short = tls.transit_mask(lc_data_short["time"].to_numpy(), row["Period (days)"],
                                                           row["Duration (hours)"] / 24, row["Epoch (BJD)"] - 2457000.0)
                             tags_series_short[mask_short] = prepare_tic_input.label
-                        mask_long = tls.transit_mask(lc_long.time.value, row["Period (days)"], row["Duration (hours)"] / 24,
+                        mask_long = tls.transit_mask(lc_data_long["time"].to_numpy(), row["Period (days)"], row["Duration (hours)"] / 24,
                                                      row["Epoch (BJD)"] - 2457000.0)
                         tags_series_long[mask_long] = prepare_tic_input.label
-            if lc_short is not None:
-                lc_classified_short = pd.DataFrame.from_dict(
-                    {"time": lc_short.time.value, "flux": lc_short.flux.value, "tag": tags_series_short})
-                lc_classified_short.to_csv(target_dir + "lc_classified_short.csv")
-            lc_classified_long = pd.DataFrame.from_dict(
-                {"time": lc_long.time.value, "flux": lc_long.flux.value, "tag": tags_series_long})
-            lc_classified_long.to_csv(target_dir + "lc_classified_long.csv")
+            target_ois_df.to_csv(target_dir + "/ois.csv")
+            if lc_data_short is not None:
+                lc_data_short["tag"] = tags_series_short
+                lc_data_short.to_csv(target_dir + "time_series_short.csv")
+            lc_data_long["tag"] = tags_series_long
+            lc_data_long.to_csv(target_dir + "time_series_long.csv")
             # TODO store folded light curves -with local and global views-(masking previous candidates?)
         except Exception as e:
             failed_target = tic_id
@@ -386,7 +426,7 @@ class MlTrainingSetPreparer:
         inputs = []
         failed_targets = []
         for sector in np.arange(first_sector, 27, 1):
-            self.negative_dir = self.dir + "/ntp/" + str(sector)
+            self.negative_dir = self.dir + "/ntp/" + str(sector) + "/"
             if not os.path.exists(self.negative_dir):
                 os.mkdir(self.negative_dir)
             sector_url = self.SECTOR_URL.format(sector)
@@ -558,6 +598,85 @@ def get_focus_flux_model_branch():
     flux_branch = keras.Model(inputs=input, outputs=flux_branch)
     return flux_branch
 
+def get_singletransit_tpf_model():
+    video_image_width = 13
+    video_image_height = 13
+    video_image_channels = 1
+    sequences_per_video = 100
+    tpf_model_input = keras.Input(
+        shape=(video_image_height, video_image_width, sequences_per_video, video_image_channels),
+        name="tpf_input")
+    tpf_model = keras.layers.SpatialDropout3D(rate=0.3)(tpf_model_input)
+    tpf_model = keras.layers.Conv3D(filters=100, kernel_size=(3, 3, 5), strides=(1, 1, 3), activation="relu")(
+        tpf_model)
+    tpf_model = keras.layers.SpatialDropout3D(rate=0.2)(tpf_model)
+    tpf_model = keras.layers.Conv3D(filters=200, kernel_size=(3, 3, 5), strides=(1, 1, 3), activation="relu")(
+        tpf_model)
+    tpf_model = keras.layers.SpatialDropout3D(rate=0.1)(tpf_model)
+    tpf_model = keras.layers.MaxPooling3D(pool_size=(5, 5, 10), strides=(3, 3, 6), padding='same')(tpf_model)
+    tpf_model = keras.layers.Dense(200, activation="relu")(tpf_model)
+    tpf_model = keras.layers.Dense(100, activation="relu")(tpf_model)
+    tpf_model = keras.layers.Dense(20, activation="relu")(tpf_model)
+    tpf_model = keras.layers.Flatten()(tpf_model)
+    return keras.Model(inputs=tpf_model_input, outputs=tpf_model)
+
+def get_singletransit_motion_centroids_model():
+    mc_input = keras.Input(
+        shape=(100, 4),
+        name="motion_centroids_input")
+    mc_model = keras.layers.Conv1D(filters=50, kernel_size=3, strides=3, activation="relu", use_bias=True,
+                                   padding='same')(mc_input)
+    mc_model = keras.layers.SpatialDropout1D(rate=0.3)(mc_model)
+    mc_model = keras.layers.Conv1D(filters=100, kernel_size=5, strides=5, activation="relu", use_bias=True,
+                                   padding='same')(mc_model)
+    mc_model = keras.layers.SpatialDropout1D(rate=0.2)(mc_model)
+    mc_model = keras.layers.MaxPooling1D(pool_size=5, strides=3, padding='same')(mc_model)
+    mc_model = keras.layers.Dense(50, activation="relu")(mc_model)
+    mc_model = keras.layers.Dense(20, activation="relu")(mc_model)
+    mc_model = keras.layers.Flatten()(mc_model)
+    return keras.Model(inputs=mc_input, outputs=mc_model)
+
+def get_singletransit_bckflux_model():
+    bck_input = keras.Input(shape=(100, 1),name="bck_input")
+    bck_model = keras.layers.Conv1D(filters=25, kernel_size=2, strides=2, activation="relu", use_bias=True,
+                                   padding='same')(bck_input)
+    bck_model = keras.layers.SpatialDropout1D(rate=0.3)(bck_model)
+    bck_model = keras.layers.Conv1D(filters=50, kernel_size=3, strides=3, activation="relu", use_bias=True,
+                                   padding='same')(bck_model)
+    bck_model = keras.layers.SpatialDropout1D(rate=0.2)(bck_model)
+    bck_model = keras.layers.MaxPooling1D(pool_size=5, strides=3, padding='same')(bck_model)
+    bck_model = keras.layers.Dense(50, activation="relu")(bck_model)
+    bck_model = keras.layers.Dense(10, activation="relu")(bck_model)
+    bck_model = keras.layers.Flatten()(bck_model)
+    return keras.Model(inputs=bck_input, outputs=bck_model)
+
+def get_singletransit_flux_model():
+    bck_input = keras.Input(shape=(100, 1),name="flux_input")
+    bck_model = keras.layers.Conv1D(filters=25, kernel_size=2, strides=2, activation="relu", use_bias=True,
+                                   padding='same')(bck_input)
+    bck_model = keras.layers.SpatialDropout1D(rate=0.3)(bck_model)
+    bck_model = keras.layers.Conv1D(filters=50, kernel_size=3, strides=3, activation="relu", use_bias=True,
+                                   padding='same')(bck_model)
+    bck_model = keras.layers.SpatialDropout1D(rate=0.2)(bck_model)
+    bck_model = keras.layers.MaxPooling1D(pool_size=5, strides=3, padding='same')(bck_model)
+    bck_model = keras.layers.Dense(50, activation="relu")(bck_model)
+    bck_model = keras.layers.Dense(10, activation="relu")(bck_model)
+    bck_model = keras.layers.Flatten()(bck_model)
+    return keras.Model(inputs=bck_input, outputs=bck_model)
+
+
+def get_single_transit_model():
+    tpf_branch = get_singletransit_tpf_model()
+    mc_branch = get_singletransit_motion_centroids_model()
+    bck_branch = get_singletransit_bckflux_model()
+    flux_branch = get_singletransit_flux_model()
+    final_branch = keras.layers.concatenate([tpf_branch.output, mc_branch.output, bck_branch.output, flux_branch.output], axis=1)
+    final_branch = keras.layers.Dense(64, activation="relu", name="final-dense1")(final_branch)
+    final_branch = keras.layers.Dense(32, activation="relu", name="final-dense2")(final_branch)
+    final_branch = keras.layers.Dense(1, activation="softmax", name="final-dense-softmax")(final_branch)
+    inputs = tpf_branch.inputs + mc_branch.inputs + bck_branch.inputs + flux_branch.inputs
+    model = keras.Model(inputs=inputs, outputs=final_branch, name="mnist_model")
+    keras.utils.vis_utils.plot_model(model, "multi_input_and_output_model.png", show_shapes=True)
 
 def get_model():
     # model = Sequential()
@@ -569,7 +688,7 @@ def get_model():
     # model.add(Dense(100, activation='relu'))
     # model.add(Dense(3, activation='softmax'))
     # model.compile(loss='categorical_crossentropy', optimizer='adam', metrics=['accuracy'])
-    stellar_model_input = keras.Input(shape=(6, 1), name="stellar_model") #TODO
+    stellar_model_input = keras.Input(shape=(6, 1), name="stellar_model")
     stellar_model_branch = keras.layers.Dense(16, activation="relu", name="stellar-first")(stellar_model_input)
     stellar_model_branch = keras.layers.Dropout(rate=0.1, name="stellar-first-dropout-0.1")(stellar_model_branch)
     stellar_model_branch = keras.layers.Dense(16, activation="relu", name="stellar-refinement")(stellar_model_branch)
@@ -583,6 +702,90 @@ def get_model():
     model = keras.Model(inputs=inputs, outputs=final_branch, name="mnist_model")
     keras.utils.vis_utils.plot_model(model, "multi_input_and_output_model.png", show_shapes=True)
 
+def load_candidate_single_transits(inner_dir):
+    single_transits_dir = "training_data/single_transits/"
+    single_transits_inner_dir = single_transits_dir + inner_dir
+    if not os.path.exists(single_transits_dir):
+        os.mkdir(single_transits_dir)
+    if not os.path.exists(single_transits_inner_dir):
+        os.mkdir(single_transits_inner_dir)
+    files = os.listdir(single_transits_inner_dir)
+    files_to_process = os.listdir("training_data/" + inner_dir)
+    files_to_process.sort()
+    if len(files) > 0:
+        files.sort()
+        last_file = files[-1]
+        file_name_matches = re.search("(TIC [0-9]+)", last_file)
+        target = file_name_matches[1]
+        target_index = files_to_process.index(target) + 1
+    else:
+        target_index = 0
+    files_to_process = files_to_process[target_index:]
+    for file in files_to_process:
+        target_dir = "training_data/" + inner_dir + "/" + file
+        tpfs_short_dir = target_dir + "/tpfs_short/"
+        if not os.path.exists(tpfs_short_dir):
+            continue
+        ts_short = pd.read_csv(target_dir + "/time_series_short.csv")
+        ois = pd.read_csv(target_dir + "/ois.csv")
+        tpfs_short = []
+        for tpf_file in os.listdir(tpfs_short_dir):
+            tpfs_short.append(TessTargetPixelFile(tpfs_short_dir + "/" + tpf_file))
+        for oi in ois.iterrows():
+            initial_t0 = oi[1]["t0"]
+            duration = oi[1]["duration"] / 24 * 2
+            period = oi[1]["period"]
+            transit = 0
+            for t0 in np.arange(initial_t0, ts_short["time"].max(), period):
+                fig, axs = plt.subplots(1, 1, figsize=(16, 16), constrained_layout=True)
+                tpf_short_framed = None
+                for tpf in tpfs_short:
+                    if tpf.time[0].value < t0 and tpf.time[-1].value > t0:
+                        tpf_short_framed = tpf[(tpf.time.value > t0 - duration) & (tpf.time.value < t0 + duration)]
+                        if len(tpf_short_framed) == 0:
+                            break
+                        tpf_short_framed.plot_pixels(axs, aperture_mask=tpf_short_framed.pipeline_mask)
+                        break
+                if tpf_short_framed is None or len(tpf_short_framed) == 0:
+                    continue
+                fig.suptitle("Single Transit Analysis")
+                plt.show()
+                fig, axs = plt.subplots(4, 1, figsize=(16, 16), constrained_layout=True)
+                ts_short_framed = ts_short[(ts_short["time"] > t0 - duration) & (ts_short["time"] < t0 + duration)]
+                axs[0].scatter(ts_short_framed["time"], ts_short_framed["centroids_x"].to_numpy(), color="black")
+                axs[0].scatter(ts_short_framed["time"], ts_short_framed["motion_x"].to_numpy(), color="red")
+                axs[1].scatter(ts_short_framed["time"], ts_short_framed["centroids_y"].to_numpy(), color="black")
+                axs[1].scatter(ts_short_framed["time"], ts_short_framed["motion_y"].to_numpy(), color="red")
+                axs[2].scatter(ts_short_framed["time"], ts_short_framed["background_flux"].to_numpy(), color="blue")
+                axs[3].scatter(ts_short_framed["time"], ts_short_framed["flux"].to_numpy(), color="blue")
+                fig.suptitle("Single Transit Analysis")
+                plt.show()
+                selection = None
+                def press(key):
+                    print(f"'{key}' pressed")
+                    global selection
+                    if key == "0":
+                        selection = 0.0
+                    elif key == "1":
+                        selection = 0.25
+                    elif key == "2":
+                        selection = 0.5
+                    elif key == "3":
+                        selection = 0.75
+                    elif key == "4":
+                        selection = 1.0
+                    if selection is not None:
+                        single_transit_path = single_transits_inner_dir + "/" + file + "/S" + str(transit) + "_" + str(
+                            selection)
+                        pathlib.Path(single_transit_path).mkdir(parents=True, exist_ok=True)
+                        ts_short_framed.to_csv(single_transit_path + "/ts_short_framed.csv")
+                        tpf_short_framed.to_fits(single_transit_path + "/tpf_short_framed.fits", True)
+                        stop_listening()
+                listen_keyboard(on_press=press)
+                transit = transit + 1
+
+
+
 
 class PrepareTicInput:
     def __init__(self, dir, tic, target_ois, target_additional_ois_df, excluded_ois, label):
@@ -593,11 +796,13 @@ class PrepareTicInput:
         self.excluded_ois = excluded_ois
         self.label = label
 
-# cpus = 1
-# first_negative_sector = 1
-# ml_training_set_preparer = MlTrainingSetPreparer("training_data/", "/home/martin/")
-# #ml_training_set_preparer.prepare_positive_training_dataset(cpus)
+cpus = 1
+first_negative_sector = 1
+#ml_training_set_preparer = MlTrainingSetPreparer("training_data/", "/home/martin/")
+#ml_training_set_preparer.prepare_positive_training_dataset(cpus)
 # #ml_training_set_preparer.prepare_false_positive_training_dataset(cpus)
 # ml_training_set_preparer.prepare_negative_training_dataset(first_negative_sector, cpus)
-get_model()
+load_candidate_single_transits("tp")
+#get_model()
+#get_single_transit_model()
 #TODO prepare_negative_training_dataset(negative_dir)
