@@ -568,7 +568,7 @@ handler = logging.StreamHandler(sys.stdout)
 handler.setLevel(logging.INFO)
 handler.setFormatter(formatter)
 logger.addHandler(handler)
-target_name = "TIC 7020254"
+target_name = "TIC 308994098"
 object_dir = target_name + "_EMLS"
 lc_builder = LcBuilder()
 object_info = lc_builder.build_object_info(target_name=target_name, author=None, sectors="all", file=None, cadence=120,
@@ -588,20 +588,20 @@ ab = lc_build.star_info.ld_coefficients
 times = lc_build.lc.time.value
 flux = lc_build.lc.flux.value
 flux = wotan.flatten(times, flux, method="biweight", window_length=0.5)
-planet_radius = 11.1231951740565
-planet_period = 20.2730449095364
-planet_t0 = 1495.173992
+planet_radius = 3.23885878785658
+planet_period = 99.7839603178333
+planet_t0 = 2458352.497456 - 2457000
 
-planet_duration = 3.53471632122306 / 24
-planet_inc = 89
+planet_duration = 10.1673253978791 / 24
+planet_inc = 90
 planet_ecc = 0.01
 planet_arg_periastron = 0
 t0s = [i for i in np.arange(planet_t0, np.max(times), planet_period)]
-planet_mass = 127
+planet_mass = 30
 planet_impact_param = 0
 planet_semimajor_axis = ExoMoonLeastSquares.compute_semimajor_axis(star_mass, planet_period)
 min_radius = 1.5
-max_radius = 8
+max_radius = 2.5
 
 P1 = planet_period * u.day
 a = np.cbrt((ac.G * star_mass * u.M_sun * P1 ** 2) / (4 * np.pi ** 2)).to(u.au)
