@@ -208,7 +208,7 @@ class Sherlock:
             time = lc_build.lc.time.value
             flux = lc_build.lc.flux.value
             flux_err = lc_build.lc.flux_err.value
-            period_grid = LcbuilderHelper.calculate_period_grid(time, sherlock_target.period_min,
+            period_grid, sherlock_target.oversampling = LcbuilderHelper.calculate_period_grid(time, sherlock_target.period_min,
                                                                 sherlock_target.period_max,
                                                                 sherlock_target.oversampling,
                                                                 lc_build.star_info, lc_build.transits_min_count)
@@ -672,6 +672,7 @@ class Sherlock:
     def __adjust_transit(self, sherlock_target, time, lc, star_info, transits_min_count, run_results, report, cadence,
                          period_grid, detrend_source_period):
         oversampling = sherlock_target.oversampling
+            #TODO calculate pesimistic oversampling comparing length of curve (
         model = tls.transitleastsquares(time, lc)
         power_args = {"transit_template": sherlock_target.fit_method, "period_min": sherlock_target.period_min,
                       "period_max": sherlock_target.period_max, "n_transits_min": transits_min_count,
